#!/bin/sh
# $Id: t-cltbind-6.sh,v 1.2 2011/05/18 21:23:35 ca Exp $
# Copyright (c) 2011 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test client IP address binding, selection based on IP= in MAIL
# ----------------------------------------
#

SD=`dirname $0`
THISDIR=`pwd`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

chkcto smtpc COMMERCIAL
chkcto smtpc SC_SESS_FEATS
chkcto smtps SMI_MAILEXT

if test X"`uname`" != "XOSF1"
then
while getopts S:s:V FLAG
do
  case "${FLAG}" in
    S) SKIP="${OPTARG}";;
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

getip2 2

cd ../qmgr || exit 1

${CAT} > ${SMXCNF} <<EOF
smtpc { Log_Level = 13; remote_port=${SNKPORT}; wait_for_server=4;
}
smtps {
	Log_Level = 13;
	daemon_address = localhost:${SRVPORT};
	wait_for_server=4;
}
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
	smtpc {
		source bind0 { ips = { $H2 } }
	}
}
smar { Log_Level = 12; nameserver = ${NS}; }
EOF

# create mailertable (save old version?)
${CAT} > ${MT} <<EOF
local.dom	lmtp:
some.dom	[127.0.0.1]
EOF

# ----------------------------------------
# test: send mail, check whether connect from smtpc to sink comes from right
# address based on IP=
# qmgr has to retry
TEST=1
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

CLTIP=1.2.3.4
${CAT} > ${QMGRCONF} <<EOF
smtpc_conf_cltaddr:${CLTIP}	source=bind0;
EOF
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -Ssender@some.dom -Ruser@some.dom -E 1 -X 2 -A 1 -Z "smtpc2" -d 5 -Y "-6 M -d 8" -c "-M IP=${CLTIP}" -Q "-T D=2" -s 20
then
  CNT=`grep -c "connection from $H2" ${SRVL}`
  if test $CNT -ne 2
  then
    ERR=true
    $VERBOSE && echo "$0: missing connection from $H2"
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   # test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: send mails, check whether connect from smtpc to sink comes from right
# address based on client IP address
TEST=2
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

${CAT} > ${QMGRCONF} <<EOF
smtpc_conf_cltaddr:127.0.0.1	source=bind0;
EOF
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -Ssender@some.dom -Ruser@some.dom -E 1 -X 2 -A 1 -Z "smtpc2" -d 5 -Y "-6 M -d 8" -Q "-T D=2" -s 20
then
  CNT=`grep -c "connection from $H2" ${SRVL}`
  if test $CNT -ne 2
  then
    ERR=true
    $VERBOSE && echo "$0: missing connection from $H2"
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
