#!/bin/sh
# $Id: t-overload-0.sh,v 1.4 2011/02/04 23:30:40 ca Exp $
# Copyright (c) 2011 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# overload the system and make sure smtps keep responding and recovers.
# ----------------------------------------
#

if test "X"${MTA_TIMING} = "X"
then
  echo "SKIPPED: $0: MTA_TIMING is not set"
  exit 0
fi

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

OUT="t-schedtmo-0.out"

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || exit 1

# create directory to pass socket
test -d smtps || mkdir smtps

rm -f ${QMGRCONFMAP} ${MTMAP} ${OUT}

# ----------------------------------------
${CAT} >${SMXCNF} <<EOF
smtps { log_level = 10; pass_fd_socket = smtps/smtpsfd; wait_for_server=4;
	listen_socket { type=inet; port=${SRVPORT}; }
	start_action = pass; user = ${USER};
	path = ../smtps/smtps;
	arguments = "smtps -f ${SMXCNF} ${MTA_SERVER_OPTIONS}";
	threads_max = 8;
}
smtpc { log_level = 9; remote_port = ${SNKPORT}; wait_for_server=4;
	connect_only_to = 127.0.0.1;
	io_timeout=60s;
	start_action = wait; user = ${USER};
	path = "../smtpc/smtpc"; arguments = "smtpc -f ${SMXCNF}";
}
qmgr { log_level = 12; wait_for_server=4; wait_for_client=4; 
	retry_delay_min = 5s;
	retry_delay_max = 1m;
	delivery_timeout=30s;
	AQ_entries_max=32;
	smtpc { initial_connections=1; connections_max=2; }
	start_action = wait; user = ${USER};
	path = "../qmgr/qmgr"; arguments = "qmgr -f ${SMXCNF} -d 3";
}
smar { log_level = 9;
	start_action = wait; user = ${USER};
	path = "../smar/smar"; arguments = "smar -f ${SMXCNF}";
}
EOF

${CAT} > ${MT} <<EOF
dom1.tld	[10.1.1.1]
dom2.tld	[10.1.1.2]
EOF

# ----------------------------------------
# send many msgs; smtps will be throttled due to small AQ size and slow sink
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
clearlogs
N=50

if ${SHELL} ${SD}/mcpmts.sh -w 4 -Y '-w4' -c "-3 -4 400 -s 1000 -m $N -Y dom2.tld -F dom1.tld -xx" -N -A $N -E $N -Z smtpc2 -1 -z
then
  :
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi


# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  rm -f ${MTMAP} ${OUT}
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
