/*
 * Copyright (c) 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 * $Id: tlscnfs.h,v 1.34 2014/03/04 20:27:09 ca Exp $
 */

#ifndef SM_TLSCNFS_H
#define SM_TLSCNFS_H 1

#include "sm/generic.h"
#include "sm/sm-conf.h"
#include "sm/mta.h"
#include "sm/cnf.h"

#if MTA_USE_TLS
#include "sm/tlsstr.h"
#include <openssl/ssl.h>

#if SM_TLSCNFDEF
#define EXTERN
#else
#define EXTERN extern
#endif

#if SM_SSCNFDEF
# define tls_defs ss_tls_defs
# define dhbits_definitions ss_dhbits_definitions
# define dhpfile_definitions ss_dhpfile_definitions
# define dhparam_definitions ss_dhparam_definitions
# define tls_op_names ss_tls_op_names
# define vrfy_depth_constraints ss_vrfy_depth_constraints
#endif

EXTERN
sm_conf_definition_T const
tls_op_names[]
#if SM_TLSCNFDEF
=
{
#ifdef SSL_OP_MICROSOFT_SESS_ID_BUG
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_MICROSOFT_SESS_ID_BUG",	sm_conf_type_choice_value,
	SSL_OP_MICROSOFT_SESS_ID_BUG,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_MICROSOFT_SESS_ID_BUG"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NETSCAPE_CHALLENGE_BUG
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NETSCAPE_CHALLENGE_BUG",	sm_conf_type_choice_value,
	SSL_OP_NETSCAPE_CHALLENGE_BUG,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NETSCAPE_CHALLENGE_BUG"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_LEGACY_SERVER_CONNECT
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_LEGACY_SERVER_CONNECT",	sm_conf_type_choice_value,
	SSL_OP_LEGACY_SERVER_CONNECT,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_LEGACY_SERVER_CONNECT"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG",	sm_conf_type_choice_value,
	SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG",	sm_conf_type_choice_value,
	SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER",	sm_conf_type_choice_value,
	SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_MSIE_SSLV2_RSA_PADDING
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_MSIE_SSLV2_RSA_PADDING",	sm_conf_type_choice_value,
	SSL_OP_MSIE_SSLV2_RSA_PADDING,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_MSIE_SSLV2_RSA_PADDING"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_SSLEAY_080_CLIENT_DH_BUG
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_SSLEAY_080_CLIENT_DH_BUG",	sm_conf_type_choice_value,
	SSL_OP_SSLEAY_080_CLIENT_DH_BUG,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_SSLEAY_080_CLIENT_DH_BUG"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_TLS_D5_BUG
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_TLS_D5_BUG",	sm_conf_type_choice_value,
	SSL_OP_TLS_D5_BUG,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_TLS_D5_BUG"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_TLS_BLOCK_PADDING_BUG
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_TLS_BLOCK_PADDING_BUG",	sm_conf_type_choice_value,
	SSL_OP_TLS_BLOCK_PADDING_BUG,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_TLS_BLOCK_PADDING_BUG"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS",	sm_conf_type_choice_value,
	SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_ALL
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_ALL",	sm_conf_type_choice_value,
	SSL_OP_ALL,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_ALL"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NO_QUERY_MTU
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NO_QUERY_MTU",	sm_conf_type_choice_value,
	SSL_OP_NO_QUERY_MTU,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NO_QUERY_MTU"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_COOKIE_EXCHANGE
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_COOKIE_EXCHANGE",	sm_conf_type_choice_value,
	SSL_OP_COOKIE_EXCHANGE,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_COOKIE_EXCHANGE"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NO_TICKET
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NO_TICKET",	sm_conf_type_choice_value,
	SSL_OP_NO_TICKET,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NO_TICKET"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_CISCO_ANYCONNECT
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_CISCO_ANYCONNECT",	sm_conf_type_choice_value,
	SSL_OP_CISCO_ANYCONNECT,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_CISCO_ANYCONNECT"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION",	sm_conf_type_choice_value,
	SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NO_COMPRESSION
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NO_COMPRESSION",	sm_conf_type_choice_value,
	SSL_OP_NO_COMPRESSION,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NO_COMPRESSION"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION",	sm_conf_type_choice_value,
	SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_SINGLE_ECDH_USE
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_SINGLE_ECDH_USE",	sm_conf_type_choice_value,
	SSL_OP_SINGLE_ECDH_USE,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_SINGLE_ECDH_USE"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_SINGLE_DH_USE
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_SINGLE_DH_USE",	sm_conf_type_choice_value,
	SSL_OP_SINGLE_DH_USE,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_SINGLE_DH_USE"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_EPHEMERAL_RSA
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_EPHEMERAL_RSA",	sm_conf_type_choice_value,
	SSL_OP_EPHEMERAL_RSA,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_EPHEMERAL_RSA"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_CIPHER_SERVER_PREFERENCE
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_CIPHER_SERVER_PREFERENCE",	sm_conf_type_choice_value,
	SSL_OP_CIPHER_SERVER_PREFERENCE,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_CIPHER_SERVER_PREFERENCE"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_TLS_ROLLBACK_BUG
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_TLS_ROLLBACK_BUG",	sm_conf_type_choice_value,
	SSL_OP_TLS_ROLLBACK_BUG,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_TLS_ROLLBACK_BUG"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NO_SSLv2
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NO_SSLv2",	sm_conf_type_choice_value,
	SSL_OP_NO_SSLv2,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NO_SSLv2"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NO_SSLv3
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NO_SSLv3",	sm_conf_type_choice_value,
	SSL_OP_NO_SSLv3,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NO_SSLv3"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NO_TLSv1
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NO_TLSv1",	sm_conf_type_choice_value,
	SSL_OP_NO_TLSv1,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NO_TLSv1"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_PKCS1_CHECK_1
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_PKCS1_CHECK_1",	sm_conf_type_choice_value,
	SSL_OP_PKCS1_CHECK_1,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_PKCS1_CHECK_1"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_PKCS1_CHECK_2
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_PKCS1_CHECK_2",	sm_conf_type_choice_value,
	SSL_OP_PKCS1_CHECK_2,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_PKCS1_CHECK_2"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NETSCAPE_CA_DN_BUG
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NETSCAPE_CA_DN_BUG",	sm_conf_type_choice_value,
	SSL_OP_NETSCAPE_CA_DN_BUG,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NETSCAPE_CA_DN_BUG"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NETSCAPE_DEMO_CIPHER_CHANGE_BUG
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NETSCAPE_DEMO_CIPHER_CHANGE_BUG",	sm_conf_type_choice_value,
	SSL_OP_NETSCAPE_DEMO_CIPHER_CHANGE_BUG,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NETSCAPE_DEMO_CIPHER_CHANGE_BUG"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_CRYPTOPRO_TLSEXT_BUG
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_CRYPTOPRO_TLSEXT_BUG",	sm_conf_type_choice_value,
	SSL_OP_CRYPTOPRO_TLSEXT_BUG,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_CRYPTOPRO_TLSEXT_BUG"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NO_TLSv1_1
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NO_TLSv1_1",	sm_conf_type_choice_value,
	SSL_OP_NO_TLSv1_1,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NO_TLSv1_1"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
#ifdef SSL_OP_NO_TLSv1_2
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_NO_TLSv1_2",	sm_conf_type_choice_value,
	SSL_OP_NO_TLSv1_2,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_NO_TLSv1_2"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 

#ifdef SSL_OP_SAFARI_ECDHE_ECDSA_BUG
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_SAFARI_ECDHE_ECDSA_BUG",	sm_conf_type_choice_value,
	SSL_OP_SAFARI_ECDHE_ECDSA_BUG,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_SAFARI_ECDHE_ECDSA_BUG"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 

/*
template
:'a,.s/SSL_OP_//g
#ifdef SSL_OP_
{ SM_CONF_DEF_MAGIC,
	"SSL_OP_",	sm_conf_type_choice_value,
	SSL_OP_,
	0, NULL, 0, NULL, NULL, NULL,
	"SSL_OP_"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif 
*/

CONF_SENTINEL
}
#endif /* SM_TLSCNFDEF */
;

#if SM_SSCNFDEF
EXTERN
sm_conf_definition_T const
ss_tls_flag_names[]
#if SM_TLSCNFDEF
=
{
{ SM_CONF_DEF_MAGIC,
	"allow_relaying_if_verified",		sm_conf_type_choice_value,
	SSC_TLS_REL_VRFY,
	0, NULL, 0, NULL, NULL, NULL,
	"allow relaying if client cert has been verified"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC,
	"check_access_map_for_relaying",	sm_conf_type_choice_value,
	SSC_TLS_REL_ACC,
	0, NULL, 0, NULL, NULL, NULL,
	"perform access map lookups to allow relaying"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC,
	"request_cert",	sm_conf_type_choice_value,
	SSC_TLS_RQST_CERT,
	0, NULL, 0, NULL, NULL, NULL,
	"request cert"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC,
	"anonymous",	sm_conf_type_choice_value,
	SSC_TLS_ANONYM,
	0, NULL, 0, NULL, NULL, NULL,
	"use anonymous ciphers without cert"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_TLSCNFDEF */
;

#endif /* SM_SSCNFDEF */

EXTERN
sm_conf_definition_T const
dhbits_definitions[]
/* should this be a choice instead? */
#if SM_TLSCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "bits", sm_conf_type_u32,
	offsetof(dhparam_T, dhbits.dhbits_bits), sizeof(unsigned int),
	NULL, 0,
	NULL, NULL, NULL,
	"bit length of predefined DH parameter"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_TLSCNFDEF */
;

EXTERN
sm_conf_definition_T const
dhpfile_definitions[]
#if SM_TLSCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "file", sm_conf_type_string,
	offsetof(dhparam_T, dhpfile.dhpfile_name), 0,
	NULL, 0,
	NULL, NULL, NULL,
	"name of file with DH parameters"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_TLSCNFDEF */
;

EXTERN
sm_conf_definition_T
dhparam_definitions[]
#if SM_TLSCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "", sm_conf_type_union_type,
	offsetof(dhparam_T, dhparam_type), sizeof(dhparam_type_T),
	NULL, 0,
	NULL, NULL, NULL,
	NULL
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, "bits", sm_conf_type_union_choice,
	DHPBITS, sizeof(dhparam_T),
	NULL, 0,
	dhbits_definitions, NULL, NULL,
	NULL
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, "file", sm_conf_type_union_choice,
	DHPFILE, sizeof(dhparam_T),
	NULL, 0,
	dhpfile_definitions, NULL, NULL,
	NULL
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_TLSCNFDEF */
;

EXTERN
sm_conf_definition_T const
vrfy_depth_constraints[]
#if SM_TLSCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "", sm_conf_type_u32_minimum,
	1,	0,
	NULL,	0,	NULL,	NULL,	NULL,
	""
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, "", sm_conf_type_u32_maximum,
	15,	0,
	NULL,	0,	NULL,	NULL,	NULL,
	""
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
    
CONF_SENTINEL
}
#endif /* SM_TLSCNFDEF */
;

EXTERN
sm_conf_definition_T
tls_defs[]
#if SM_TLSCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "cert_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_cert),	0,
	SM_MTA_CERT,
	SM_CONF_FLAG_CHK0|SM_CONF_FLAG_CHK1,	NULL,	NULL,	NULL,
	"File with certificate in PEM format"
	SM_LC_ISSET(offsetof(tls_cnf_T, tlscnf_cert_isset))
	SM_LC_SET_MAGIC(0) }
,{ SM_CONF_DEF_MAGIC, "key_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_key),	0,
	SM_MTA_KEY,
	SM_CONF_FLAG_CHK0|SM_CONF_FLAG_CHK1,	NULL,	NULL,	NULL,
	"File with private key for certificate in PEM format"
	SM_LC_ISSET(offsetof(tls_cnf_T, tlscnf_key_isset))
	SM_LC_SET_MAGIC(0) }
,{ SM_CONF_DEF_MAGIC, "DSA_cert_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_dsa_cert),	0,
	NULL,	SM_CONF_FLAG_CHK0|SM_CONF_FLAG_CHK1,	NULL,	NULL,	NULL,
	"File with DSA certificate in PEM format"
	SM_LC_ISSET(offsetof(tls_cnf_T, tlscnf_dsa_cert_isset))
	SM_LC_SET_MAGIC(0) }
,{ SM_CONF_DEF_MAGIC, "DSA_key_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_dsa_key),	0,
	NULL,	SM_CONF_FLAG_CHK0|SM_CONF_FLAG_CHK1,	NULL,	NULL,	NULL,
	"File with private key for DSA certificate in PEM format"
	SM_LC_ISSET(offsetof(tls_cnf_T, tlscnf_dsa_key_isset))
	SM_LC_SET_MAGIC(0) }
,{ SM_CONF_DEF_MAGIC, "CAcert_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_cacert),	0,
	SM_MTA_CACERT,
	0,	NULL,	NULL,	NULL,
	"File with CA certificate in PEM format"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
,{ SM_CONF_DEF_MAGIC, "CAcert_directory",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_certpath),	0,
	SM_MTA_CERTP,
	0,	NULL,	NULL,	NULL,
	"Directory with (symbolic links for) CA certificates in PEM format"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
# if SM_SSCNFDEF
,{ SM_CONF_DEF_MAGIC, "CAlist_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_calist),	0,
	SM_MTA_CACERT,
	SM_CONF_FLAG_CHK1,	NULL,	NULL,	NULL,
	"File with CA certificates in PEM format (for acceptable client certs)"
	SM_LC_ISSET(offsetof(tls_cnf_T, tlscnf_calist_isset))
	SM_LC_SET_MAGIC(0) }
# endif /* SM_SSCNFDEF */
,{ SM_CONF_DEF_MAGIC, "options",		sm_conf_type_choice,
	offsetof(tls_cnf_T, tlscnf_options),	sizeof(uint32_t),
	SM_XSTR(SSL_OP_ALL),
	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_PLURAL|SM_CONF_FLAG_C_CONST|SM_CONF_FLAG_CHK0|SM_CONF_FLAG_CHK1,
	tls_op_names, NULL,	NULL,
	"options for SSL_CTX_set_options [expert only]"
	SM_LC_ISSET(offsetof(tls_cnf_T, tlscnf_options_isset))
	SM_LC_SET_MAGIC(0) }

,{ SM_CONF_DEF_MAGIC, "DHParam", sm_conf_type_union,
	offsetof(tls_cnf_T, tlscnf_dhparam), sizeof(dhparam_type_T),
	NULL, 0, dhparam_definitions,
	NULL,	NULL,
	"DH parameters"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }

,{ SM_CONF_DEF_MAGIC, "CipherList",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_cipherlist),	0,
	NULL,
	SM_CONF_FLAG_CHK0|SM_CONF_FLAG_CHK1,	NULL,	NULL,	NULL,
	"Cipher List"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }

,{ SM_CONF_DEF_MAGIC, "Digest",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_fp_dig),	0,
	"sha1",
	SM_CONF_FLAG_CHK0|SM_CONF_FLAG_CHK1,	NULL,	NULL,	NULL,
	"Digest for cert fingerprints"
	SM_LC_ISSET(offsetof(tls_cnf_T, tlscnf_fp_dig_isset))
	SM_LC_SET_MAGIC(0) }

# if MTA_HAVE_ECDSA
,{ SM_CONF_DEF_MAGIC, "EC_cert_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_ec_cert),	0,
	NULL,	SM_CONF_FLAG_CHK1,	NULL,	NULL,	NULL,
	"File with EC certificate in PEM format"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
,{ SM_CONF_DEF_MAGIC, "EC_key_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_ec_key),	0,
	NULL,	SM_CONF_FLAG_CHK1,	NULL,	NULL,	NULL,
	"File with private key for EC certificate in PEM format"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
#  if SM_SSCNFDEF
/* this should use a check function for the curve name!!! */
,{ SM_CONF_DEF_MAGIC, "EC_curve",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_ec_curve),	0,
	NULL,	SM_CONF_FLAG_CHK1,	NULL,	NULL,	NULL,
	"Name of elliptic curve"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
#  endif /* SM_SSCNFDEF */
# endif /* MTA_HAVE_ECDSA */
# if MTA_USE_TLSFIPS
, { SM_CONF_DEF_MAGIC, "fips_mode",		sm_conf_type_u32,
	offsetof(tls_cnf_T, tlscnf_fips),	sizeof(uint32_t),
	0,	0,	NULL, NULL,	NULL,
	"FIPS mode"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
# endif

#if SM_SSCNFDEF
,{ SM_CONF_DEF_MAGIC, "flags",		sm_conf_type_choice,
	offsetof(tls_cnf_T, tlscnf_cflags),	sizeof(uint32_t),
	SM_XSTR(SSC_TLS_RQST_CERT) /* how to set the default? */,
	/*SM_CONF_FLAG_KEEP_DEFAULT|*/SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_OR
		|SM_CONF_FLAG_SECTION_DEFAULT_FROM_ANONYMOUS|SM_CONF_FLAG_PLURAL|SM_CONF_FLAG_C_CONST|SM_CONF_FLAG_CHK1,
										/* allow in session_features? */
	ss_tls_flag_names,
	NULL,	NULL,
	"flags to influence behavior related to STARTTLS [see documentation]"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
#endif /* SM_SSCNFDEF */

,{ SM_CONF_DEF_MAGIC, "cache_size",		sm_conf_type_u32,
	offsetof(tls_cnf_T, tlscnf_cache_size),	sizeof(uint),
	NULL,	0,	NULL,	NULL,	NULL,
	"size of TLS session cache"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }

,{ SM_CONF_DEF_MAGIC, "cache_timeout",		sm_conf_type_u32,
	offsetof(tls_cnf_T, tlscnf_cache_tmout),	sizeof(uint),
	NULL,	0,	sm_conf_time_suffixes,
	NULL,	NULL,
	"timeout for TLS session cache"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }

#if SM_SSCNFDEF
,{ SM_CONF_DEF_MAGIC, "requirements_violation",		sm_conf_type_choice,
	offsetof(tls_cnf_T, tlscnf_tlsreqfail),	sizeof(uint),
	"abort",
	SM_CONF_FLAG_KEEP_DEFAULT|SM_CONF_FLAG_CHK1,
	tlsreq_violation,
	NULL,	NULL,
	"how to treat TLS requirements violation"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
#endif /* SM_SSCNFDEF */

,{ SM_CONF_DEF_MAGIC, "verify_depth",		sm_conf_type_u32,
	offsetof(tls_cnf_T, tlscnf_vrfy_depth),	sizeof(uint),
	NULL,	0,	vrfy_depth_constraints,	NULL,	NULL,
	"maximum checked depth of certificate chain for verification"
	SM_LC_ISSET(offsetof(tls_cnf_T, tlscnf_vrfy_depth_isset))
	SM_LC_SET_MAGIC(0) }

,{ SM_CONF_DEF_MAGIC, "CRL_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_crlfile),	0,
	NULL,
	0,	NULL,	NULL,	NULL,
	"Certificate Revocation List file"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
,{ SM_CONF_DEF_MAGIC, "CRL_directory",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_crlpath),	0,
	NULL,
	0,	NULL,	NULL,	NULL,
	"Certificate Revocation List directory"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }

#if MTA_TLS_DEBUG
,{ SM_CONF_DEF_MAGIC, "debug",		sm_conf_type_u32,
	offsetof(tls_cnf_T, tlscnf_debug),	sizeof(uint32_t),
	NULL,
	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_OR|SM_CONF_FLAG_PLURAL|SM_CONF_FLAG_CHK0|SM_CONF_FLAG_CHK1,
	NULL,	NULL,	NULL,
	"debug options [experts only]"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
#endif /* MTA_TLS_DEBUG */

,CONF_SENTINEL
}
#endif /* SM_TLSCNFDEF */
;

#undef EXTERN

#endif /* MTA_USE_TLS */

#endif /* SM_TLSCNFS_H */
