/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-conf-9.c,v 1.7 2013/10/23 22:13:41 ca Exp $")

#include "sm/string.h"
#include "sm/sm-conf.h"
#include "sm/net.h"

#include "sm/io.h"
#include "sm/sm-conf-prt.h"

static int Verbose = 0;
enum vehicle_type_E
{
	AIRPLANE	= 1,
	BOAT		= 2,
	CAR		= 3
};
typedef enum vehicle_type_E vehicle_type_T;

typedef union vehicle_U
{
	vehicle_type_T type;

	struct {
		vehicle_type_T	air_type;
		char			*air_flight;
		int			air_passengers;
	} airplane;

	struct {
		vehicle_type_T	boat_type;
		int			boat_knots;
		char			*boat_name;
	} boat;

	struct {
		vehicle_type_T	car_type;
		int			car_mpg;
		char			*car_plate;
	} car;

} vehicle_T;

#define VS_MAX	8

typedef struct
{
	vehicle_T		fleet[VS_MAX];
	unsigned int	fleet_n;
} structure;

sm_conf_definition_T
const vehicle_types[]
=
{
{ SM_CONF_DEF_MAGIC,
  "airplane",   sm_conf_type_choice_value, AIRPLANE
    , 0, NULL, 0, NULL, NULL, NULL, NULL
    SM_LC_NO_ISSET  SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC,
  "boat",   sm_conf_type_choice_value, BOAT
    , 0, NULL, 0, NULL, NULL, NULL, NULL
    SM_LC_NO_ISSET  SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC,
  "car",  sm_conf_type_choice_value, CAR
    , 0, NULL, 0, NULL, NULL, NULL, NULL
    SM_LC_NO_ISSET  SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
;


static sm_conf_definition_T const
air_definitions[] =
{
{ SM_CONF_DEF_MAGIC,
  "type", sm_conf_type_choice,
	offsetof(vehicle_T, type), sizeof(vehicle_type_T),
	NULL, SM_CONF_FLAG_STRICTLY_REQUIRED, vehicle_types, NULL, NULL,
	"vehicle type"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

	{ SM_CONF_DEF_MAGIC, "flight", sm_conf_type_string,
		offsetof(vehicle_T, airplane.air_flight), 0, NULL
	},

	{ SM_CONF_DEF_MAGIC, "passengers", sm_conf_type_u32,
		offsetof(vehicle_T, airplane.air_passengers),
		sizeof(unsigned int), "1"
	},

CONF_SENTINEL
};

static sm_conf_definition_T const
boat_definitions[] =
{
{ SM_CONF_DEF_MAGIC,
  "type", sm_conf_type_choice,
	offsetof(vehicle_T, type), sizeof(vehicle_type_T),
	NULL, SM_CONF_FLAG_STRICTLY_REQUIRED, vehicle_types, NULL, NULL,
	"vehicle type"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

	{ SM_CONF_DEF_MAGIC, "name", sm_conf_type_string,
		offsetof(vehicle_T, boat.boat_name), 0, NULL
	},

	{ SM_CONF_DEF_MAGIC, "knots", sm_conf_type_u32,
		offsetof(vehicle_T, boat.boat_knots),
		sizeof(unsigned int),  NULL
	},

CONF_SENTINEL
};

static sm_conf_definition_T const
car_definitions[] =
{
{ SM_CONF_DEF_MAGIC,
  "type", sm_conf_type_choice,
	offsetof(vehicle_T, type), sizeof(vehicle_type_T),
	NULL, SM_CONF_FLAG_STRICTLY_REQUIRED, vehicle_types, NULL, NULL,
	"vehicle type"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

	{ SM_CONF_DEF_MAGIC, "plate", sm_conf_type_string,
		offsetof(vehicle_T, car.car_plate), 0, NULL
	},

	{ SM_CONF_DEF_MAGIC, "mpg", sm_conf_type_u32,
		offsetof(vehicle_T, car.car_mpg),
		sizeof(unsigned int),  NULL
	},

CONF_SENTINEL
};

static sm_conf_definition_T
vehicle_definitions[] =
{

	{ SM_CONF_DEF_MAGIC, "", sm_conf_type_union_type,
		offsetof(vehicle_T, type), sizeof(vehicle_type_T),
		NULL, 0, NULL
	},

	{ SM_CONF_DEF_MAGIC, "type", sm_conf_type_union_choice,
		/* offset = type tag */ AIRPLANE,
		sizeof(vehicle_T),  "airplane", 0, air_definitions
	},

	{ SM_CONF_DEF_MAGIC, "type", sm_conf_type_union_choice,
		/* offset = type tag */ BOAT,
		sizeof(vehicle_T), "boat", 0, boat_definitions
	},

	{ SM_CONF_DEF_MAGIC, "type", sm_conf_type_union_choice,
		/* offset = type tag */ CAR,
		sizeof(vehicle_T),  "car", 0, car_definitions
	},

CONF_SENTINEL
};

static sm_conf_definition_T
fleet_definitions[] =
{

	{ SM_CONF_DEF_MAGIC, "vehicle", sm_conf_type_union,
		offsetof(structure, fleet), sizeof(vehicle_T),
		NULL, 0, vehicle_definitions, NULL, NULL, "fleet of vehicles"
	},

	{ SM_CONF_DEF_MAGIC, "n", sm_conf_type_array_n,
		offsetof(structure, fleet_n), sizeof(unsigned int),
		NULL, 0, vehicle_definitions, NULL, NULL,
		"number of vehicles in the fleet"
	},

CONF_SENTINEL
};

static sm_conf_definition_T const
definitions[] =
{

	{ SM_CONF_DEF_MAGIC, "vehicle", sm_conf_type_array,
		0, VS_MAX, NULL, SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_FLAT,
		fleet_definitions, NULL, NULL,
		"A fleet with a fixed number of vehicles."
	},

CONF_SENTINEL
};

static void
print_vehicle(vehicle_T const v)
{
	switch (v.type) {
	  case AIRPLANE:
		sm_io_fprintf(smioout, "airplane { flight=%s passengers=%d }\n",
			v.airplane.air_flight == NULL
				? "(null)" : v.airplane.air_flight,
			v.airplane.air_passengers);
		break;
	  case BOAT:
		sm_io_fprintf(smioout, "boat { knots=%d name=%s }\n",
			v.boat.boat_knots,
			v.boat.boat_name == NULL
				? "(null)" : v.boat.boat_name);
		break;
	  case CAR:
		sm_io_fprintf(smioout, "car { mpg=%d plate=%s }\n",
			v.car.car_mpg,
			v.car.car_plate == NULL ? "(null)" : v.car.car_plate);
		break;

	  case 0:
		sm_io_fprintf(smioout, "(empty)\n");
		break;

	  default:
		sm_io_fprintf(smioout, "<unexpected type %d>\n", (int)v.type);
		break;
	}
}

static void
print_structure(structure *s)
{
	int i;

	sm_io_fprintf(smioout, "fleet: [%d]\n", s->fleet_n);
	for (i = 0; i < s->fleet_n; i++) {
		sm_io_fprintf(smioout, "[%d] ", i);
		print_vehicle(s->fleet[i]);
	}

	/* sm_io_putc(smioout, '\n'); */
	sm_io_flush(smioout);
}

static int
process(char const *name, FILE *fp)
{
	sm_conf_T		*stream;
	int			err;
	structure		s;

	if (((stream = sm_conf_new(name ? name : "*stdin*"))) == NULL) {
		fprintf(stderr, "error -- sm_conf_new() returns NULL!\n");
		return 1;
	}
	if ((err = sm_conf_read_FILE(stream, name, fp)) != 0) {
		char buf[SM_CONF_ERROR_BUFFER_SIZE];
		char const *e = NULL;

		fprintf(stderr, "%s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));

		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			fprintf(stderr, "%s\n", e);

		sm_conf_destroy(stream);
		return 2;
	}

	memset(&s, 0, sizeof(s));
	err = sm_conf_scan(stream, definitions, 0, &s);
	if (err != 0) {
		char buf[SM_CONF_ERROR_BUFFER_SIZE];
		char const *e = NULL;

		fprintf(stderr, "(while scanning) %s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));

		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			fprintf(stderr, "%s\n", e);

		sm_conf_destroy(stream);
		return 3;
	}

	if (Verbose > 0) {
		sm_io_fprintf(smioout, "configuration\n");
		sm_conf_prt_conf(definitions, &s, smioout);
		sm_io_flush(smioout);
	}

	print_structure(&s);
	sm_conf_destroy(stream);

	return 0;
}

int
main(int ac, char **av)
{
	int ai, r;

	while ((r = getopt(ac, av, "V")) != -1) {
		switch (r) {
		  case 'V':
			Verbose++;
			break;
		  default:
			/* usage(av[0]); */
			return 1;
		}
	}

	ac -= optind;
	av += optind;

	if (ac == 0)
		return process("*stdin*", stdin);

	for (ai = 0; ai < ac; ai++) {
		int ret = process(av[ai], NULL);
		if (ret != 0)
			return ret;
	}
	return 0;
}
