/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: rcbput3uintn.c,v 1.2 2011/01/30 21:49:06 ca Exp $")
#include "sm/limits.h"
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/magic.h"
#include "sm/memops.h"
#include "sm/rpool.h"
#include "sm/rcb.h"
#include "sm/str-int.h"
#include "sm/str2rcb.h"
#include "sm/reccom.h"

/*
**  SM_RCB_PUT3UINTn -- Append two ints and one buf to the end of a sm_rcb_P
**
**	Parameters:
**		rcb -- sm_rcb_P object to append onto.
**		n1 -- value to append.
**		n2 -- value to append.
**		buf -- buf to append.
**		len -- len of buf
**
**	Returns:
**		usual sm_error code; ENOMEM, SM_E_OVFLW_SC, SM_E_OVFLW_NS
**
**	Last code review:
**	Last code change:
*/

sm_ret_T
sm_rcb_put3uintn(sm_rcb_P rcb, uint32_t n1, uint32_t n2, const void *buf, uint len)
{
	uint nsize, l;

	SM_IS_RCB(rcb);

	l = sizeof(uint32_t) * 2 + len;
	nsize = rcb->sm_rcb_len + l;

	/* overflow? */
	if (nsize <= rcb->sm_rcb_len)
		return sm_error_perm(SM_EM_RECCOM, SM_E_OVFLW_SC);

	/* do we really want to do this? */
	if (rcb->sm_rcb_rw >= 0 && (int)l > rcb->sm_rcb_rw)
		return sm_error_perm(SM_EM_RECCOM, SM_E_FULL);
	if (nsize > rcb->sm_rcb_size)
		SM_STR_INCREASE_R(rcb, nsize);
	sm_uint32_2buf(n1, rcb->sm_rcb_base + rcb->sm_rcb_len);
	rcb->sm_rcb_len += sizeof(uint32_t);
	sm_uint32_2buf(n2, rcb->sm_rcb_base + rcb->sm_rcb_len);
	rcb->sm_rcb_len += sizeof(uint32_t);
	sm_memcpy(rcb->sm_rcb_base + rcb->sm_rcb_len, buf, len);
	rcb->sm_rcb_len += len;
	if (rcb->sm_rcb_rw >= 0)
		rcb->sm_rcb_rw -= l;
	return SM_SUCCESS;
}
