/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: stsockconn.c,v 1.2 2010/10/03 03:18:03 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/memops.h"
#include "statethreads/st.h"
#include "sm/rcbst.h"
#include "sm/fcntl.h"
#include "sm/socket.h"
#include "sm/sockcnf.h"
#include "sm/stsock.h"

/*
**  ST_SOCKADDR_CONNECT -- connect to a socket
**
**	Parameters:
**		sockaddr -- socket address
**		tmo -- timeout
**		fd -- pointer to net fd (output)
**
**	Returns:
**		usual return code
*/

sm_ret_T
st_sockaddr_connect(sm_sockaddr_T sm_sockaddr, st_utime_t tmo, st_netfd_t *fd)
{
	int sockfd;
	st_netfd_t nfd;
	sockaddr_len_T sock_size;

	SM_REQUIRE(fd != NULL);
	*fd = NULL;

	switch (sm_sockaddr.sa.sa_family) {
	  case AF_INET:
		sock_size = sizeof(sm_sockaddr.sin);
		break;

#if MTA_NETINET6
	  case AF_INET6:
		sock_size = sizeof(sm_sockaddr.sin6);
		break;
#endif
	  case AF_UNIX:
		sock_size = sizeof(sm_sockaddr.sunix);
		break;
	  default:
		return sm_error_perm(SM_EM_STTHRIO, EINVAL);
	}

	sockfd = socket(sm_sockaddr.sa.sa_family, SOCK_STREAM, 0);
	if (sockfd == -1)
		return sm_error_perm(SM_EM_STTHRIO, errno);

	nfd = st_netfd_open_socket(sockfd);
	if (nfd == NULL)
		return sm_error_perm(SM_EM_STTHRIO, errno);
	if (st_connect(nfd, &sm_sockaddr.sa, sock_size, tmo) == -1) {
		int err;

		err = errno;
		(void) st_netfd_close(nfd);
		return sm_error_perm(SM_EM_STTHRIO, err);
	}

	*fd = nfd;
	return SM_SUCCESS;
}
