#!/bin/sh
# $Id: t-rcptcnf-0.sh,v 1.10 2014/05/29 14:36:58 ca Exp $
#
# ------------------------------------------------------------
# Description:
# Script to test "full" greylisting
# Uses smar, t-access-0, access map, t-mm, startsmar.sh
# ------------------------------------------------------------
# Note: smar and t-access-0 are running asynchronous, hence
# there must be no expected order in the output file.
#

# this is a "slow" test?
if test X"${MTA_NO_SLOW_TEST}" != "X"
then
  echo "SKIPPED: $0: MTA_NO_SLOW_TEST is set"
  exit 0
fi

SD=`dirname $0`
TPRG=./t-access-0
if test "${SD}" = "."
then
   SD=../smar
fi
. ${SD}/common.sh

CONF=grey.conf

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

rm -f ${ALIMAP} ${ACCMAP} ${SOCK} ${PIDS}

# create access map for tests
#cltaddr:127.0.0.1	ok
${CAT} >${ACC} <<EOF
cltaddr:127.	ok
smtps_rcpt_conf:rcpt@example.org	flags=greylisting;
EOF
${VALGRIND} ${MM} -t'	' -F ${ACCMAP} < ${ACC}
if test $? -ne 0
then
  echo t-mm failed
  exit 1
fi

# 
${CAT} > ${CONF} <<EOF
smar {
greylisting { grey_wait=10s; grey_expire=20s; white_timeout=15s; }
}
EOF

trap killit 2

rm -f grey_grey_m.db grey_grey_s.db

# start SMAR
${SHELL} ${SD}/startsmar.sh -f ${CONF} -o '-d 9' ${SMAROPTS} || exit 1
rm -f ${OUT} ${IP} ${OK}

NOW=10000

# 
TEST=1
# ----------------------------------------
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
${VALGRIND} ${TPRG} -t ${NOW} -VVV -4 '10.1.2.3' -P '<sender@example.com>' '<rcpt@example.org>' > ${OUT} 2>&1 || ERRS=`expr ${ERRS} + 1 `
grep '^react=' ${OUT} | sort > ${IP}
sort > ${OK} <<EOF
react=statt, ret=0, statt='450 4.7.0 Come back later.
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed; part 1"
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

NOW=`expr ${NOW} + 1 `
${VALGRIND} ${TPRG} -t ${NOW} -4 '10.1.2.3' -P '<sender@example.com>' '<rcpt@example.org>' > ${OUT} 2>&1 || ERRS=`expr ${ERRS} + 1 `
grep '^react=' ${OUT} > ${IP}
${CAT} > ${OK} <<EOF
react=statt, ret=0, statt='450 4.7.0 Come back later.
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed; part 2"
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

NOW=`expr ${NOW} + 11 `

${VALGRIND} ${TPRG} -t ${NOW} -4 '10.1.2.3' -G '<sender@example.com>' '<rcpt@example.org>' > ${OUT} 2>&1 || ERRS=`expr ${ERRS} + 1 `
grep '^react=' ${OUT} > ${IP}
${CAT} > ${OK} <<EOF
react=no_match
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed; part 3"
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

NOW=`expr ${NOW} + 20 `

# grey record expired, start over
${VALGRIND} ${TPRG} -t ${NOW} -4 '10.1.2.3' -P '<sender@example.com>' '<rcpt@example.org>' > ${OUT} 2>&1 || ERRS=`expr ${ERRS} + 1 `
grep '^react=' ${OUT} > ${IP}
${CAT} > ${OK} <<EOF
react=statt, ret=0, statt='450 4.7.0 Come back later.
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed; part 4"
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi


fi

# ----------------
# stop SMAR
for i in `${CAT} ${PIDS}`
do
  kill ${i}
done
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi
sleep 3

rm -f ${OUT} ${IP} ${OK} ${CONF}
rm -f grey_grey_m.db grey_grey_s.db

exit ${ERRS}
